@@ -182,6 +182,10 @@ typecast_parse_time(const char* s, const char** t, Py_ssize_t* len,
 #include "psycopg/typecast_array.c"
 #include "psycopg/typecast_builtins.c"
 
+#define typecast_PYDATETIMEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_PYDATEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_PYTIMEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_PYINTERVALARRAY_cast typecast_GENERIC_ARRAY_cast
 
 /* a list of initializers, used to make the typecasters accessible anyway */
 static typecastObject_initlist typecast_pydatetime[] = {
@@ -189,16 +193,29 @@ static typecastObject_initlist typecast_pydatetime[] = {
     {"PYTIME", typecast_TIME_types, typecast_PYTIME_cast},
     {"PYDATE", typecast_DATE_types, typecast_PYDATE_cast},
     {"PYINTERVAL", typecast_INTERVAL_types, typecast_PYINTERVAL_cast},
+    {"PYDATETIMEARRAY", typecast_DATETIMEARRAY_types, typecast_PYDATETIMEARRAY_cast, "PYDATETIME"},
+    {"PYTIMEARRAY", typecast_TIMEARRAY_types, typecast_PYTIMEARRAY_cast, "PYTIME"},
+    {"PYDATEARRAY", typecast_DATEARRAY_types, typecast_PYDATEARRAY_cast, "PYDATE"},
+    {"PYINTERVALARRAY", typecast_INTERVALARRAY_types, typecast_PYINTERVALARRAY_cast, "PYINTERVAL"},
     {NULL, NULL, NULL}
 };
 
-/* a list of initializers, used to make the typecasters accessible anyway */
 #ifdef HAVE_MXDATETIME
+#define typecast_MXDATETIMEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_MXDATEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_MXTIMEARRAY_cast typecast_GENERIC_ARRAY_cast
+#define typecast_MXINTERVALARRAY_cast typecast_GENERIC_ARRAY_cast
+
+/* a list of initializers, used to make the typecasters accessible anyway */
 static typecastObject_initlist typecast_mxdatetime[] = {
     {"MXDATETIME", typecast_DATETIME_types, typecast_MXDATE_cast},
     {"MXTIME", typecast_TIME_types, typecast_MXTIME_cast},
     {"MXDATE", typecast_DATE_types, typecast_MXDATE_cast},
     {"MXINTERVAL", typecast_INTERVAL_types, typecast_MXINTERVAL_cast},
+    {"MXDATETIMEARRAY", typecast_DATETIMEARRAY_types, typecast_MXDATETIMEARRAY_cast, "MXDATETIME"},
+    {"MXTIMEARRAY", typecast_TIMEARRAY_types, typecast_MXTIMEARRAY_cast, "MXTIME"},
+    {"MXDATEARRAY", typecast_DATEARRAY_types, typecast_MXDATEARRAY_cast, "MXDATE"},
+    {"MXINTERVALARRAY", typecast_INTERVALARRAY_types, typecast_MXINTERVALARRAY_cast, "MXINTERVAL"},
     {NULL, NULL, NULL}
 };
 #endif
@@ -381,9 +398,12 @@ typecast_dealloc(PyObject *obj)
 {
     typecastObject *self = (typecastObject*)obj;
 
+    PyObject_GC_UnTrack(self);
+
     Py_CLEAR(self->values);
     Py_CLEAR(self->name);
     Py_CLEAR(self->pcast);
+    Py_CLEAR(self->bcast);
 
     obj->ob_type->tp_free(obj);
 }
@@ -396,6 +416,7 @@ typecast_traverse(PyObject *obj, visitproc visit, void *arg)
     Py_VISIT(self->values);
     Py_VISIT(self->name);
     Py_VISIT(self->pcast);
+    Py_VISIT(self->bcast);
     return 0;
 }
 
@@ -414,6 +435,15 @@ typecast_call(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
+    // If the string is not a string but a None value we're being called
+    // from a Python-defined caster. There is no need to convert, just
+    // return it.
+
+    if (string == Py_None) {
+        Py_INCREF(string);
+        return string;
+    }
+
     return typecast_cast(obj,
                          PyString_AsString(string), PyString_Size(string),
                          cursor);
@@ -587,7 +617,7 @@ typecast_cast(PyObject *obj, const char *str, Py_ssize_t len, PyObject *curs)
     PyObject *old, *res = NULL;
     typecastObject *self = (typecastObject *)obj;
 
-    /* we don't incref, the caster *can't* die at this point */
+    Py_INCREF(obj);
     old = ((cursorObject*)curs)->caster;
     ((cursorObject*)curs)->caster = obj;
 
@@ -602,6 +632,7 @@ typecast_cast(PyObject *obj, const char *str, Py_ssize_t len, PyObject *curs)
     }
 
     ((cursorObject*)curs)->caster = old;
+    Py_DECREF(obj);
 
     return res;
 }
